#property copyright "Copyright  2012 by MandySoft"
#property link      "http://www.2p.cz/en/forex_metatrader"
// Version: 0.2
/* Revision:
  v0.1 
    - initial version
  v0.2
    - dependency on TradingHours
    - non continous lines
*/

#include <MarketDef.mqh>
#include <MarketHelper.mqh>
#include <TZDBHelper.mqh>
#include <NY 7-9.mqh>
#include <Debug.mqh>

#define NUM_BUFFERS 4

// #property indicator_separate_window
#property indicator_chart_window
#property indicator_buffers NUM_BUFFERS
#property indicator_color1 Red
#property indicator_color2 Red
#property indicator_color3 Green
#property indicator_color4 Green
#property indicator_style1 STYLE_SOLID
#property indicator_style2 STYLE_SOLID
#property indicator_style3 STYLE_SOLID
#property indicator_style4 STYLE_SOLID
//#property indicator_width1 2
//#property indicator_width2 2

double buffInBoxLow[];
double buffInBoxHigh[];
double buffOutBoxLow[];
double buffOutBoxHigh[];

bool inInterval = false;
int lastStartIdx = -1;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init() {
   int i;
//   DebugOpen(NY_7_9_INDICATOR, NY_7_9_INDICATOR+".#.dbg");
   IndicatorBuffers(NUM_BUFFERS);
   SetIndexBuffer(NY_7_9_BOX_LOW, buffInBoxLow);
   SetIndexBuffer(NY_7_9_BOX_HIGH, buffInBoxHigh);
   SetIndexBuffer(NY_7_9_MARKET_LOW, buffOutBoxLow);
   SetIndexBuffer(NY_7_9_MARKET_HIGH, buffOutBoxHigh);
   for (i=0; i<NUM_BUFFERS; i++) {
      SetIndexStyle(i, DRAW_LINE); 
   }
   SetIndexLabel(NY_7_9_BOX_LOW, "Low");
   SetIndexLabel(NY_7_9_BOX_HIGH, "High");
   SetIndexLabel(NY_7_9_MARKET_LOW, "Low");
   SetIndexLabel(NY_7_9_MARKET_HIGH, "High");
               
   IndicatorShortName("HiLo "+GetMarketHelperInfo(MARKET_NEWYORK_07_09, MARKET_NAME_IDX));
   return(0);
}
  
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit() {
  Debug("deinit", UninitializeReason());
  DebugClose();
  return(0);
}
  
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start() {
   double sec1 = 1.0/24.0/60.0/60.0;
   int limit;
   if (Period()>PERIOD_H1) {
       Debug("Period:", Period());
      return(0);    // max.reasonable resolution is probably 1 hour
   }
   if (IndicatorCounted() == 0) {  // indicator restart, is it possible at all ?
     inInterval = false;
     lastStartIdx = -1;   
   }
   
   // normal result of (Bars - countedBars):  >=2 .. at start, =1 .. new tick, =2 .. at new candle
   int countedBars = IndicatorCounted();   
   if (countedBars < 0) {
     Debug("CounterBars:", countedBars);
     return(-1);
   }
   limit = Bars - countedBars;
   if (limit > 1) {
     Debug(IndicatorCounted(), Bars, lastStartIdx, inInterval, Low[0], High[0]);   
   }
   if (lastStartIdx >= 0) {
     lastStartIdx += limit - 1;  // limit == 1 .. rewrite candle being formed only, no bar movement!
   }
   // we cannot recount any bars at the edge because we are using global inInterval, lastStartIdx flags !
   for (int i = limit-1; i >= 0; i--) {
   
     if (IsMarketTime(MARKET_NEWYORK_07_09, Time[i])) {
       if (!inInterval) {
         inInterval = true;
         lastStartIdx = i;
         buffInBoxLow[i] = Low[i];
         buffInBoxHigh[i] = High[i];
       } else {
         int j;
         if (Low[i] < buffInBoxLow[lastStartIdx]) {
           for (j=lastStartIdx; j>=i; j--) {
             buffInBoxLow[j] = Low[i];
           }
         } else {
           buffInBoxLow[i] = buffInBoxLow[lastStartIdx];         
         }
         if (High[i] > buffInBoxHigh[lastStartIdx]) {
           for (j=lastStartIdx; j>=i; j--) {
             buffInBoxHigh[j] = High[i];
           }
         } else {
           buffInBoxHigh[i] = buffInBoxHigh[lastStartIdx];
         }
       }
       buffOutBoxLow[i] = EMPTY_VALUE;
       buffOutBoxHigh[i] = EMPTY_VALUE;
     } else if (lastStartIdx >= 0) {
       if (IsMarketTime(MARKET_NEWYORK, Time[i])) {
         buffOutBoxLow[i] = buffInBoxLow[lastStartIdx];
         buffOutBoxHigh[i] = buffInBoxHigh[lastStartIdx];
       } else {
         if (inInterval) {   /* NY session is disabled because a news but NY7-9 remained enabled, cancel it here */
           for (j=lastStartIdx; j>i; j--) {
             buffInBoxHigh[j] = EMPTY_VALUE;
             buffInBoxLow[j] = EMPTY_VALUE;
           }         
         }
         buffOutBoxLow[i] = EMPTY_VALUE;
         buffOutBoxHigh[i] = EMPTY_VALUE;
         lastStartIdx = -1;
       }
       buffInBoxLow[i] = EMPTY_VALUE;
       buffInBoxHigh[i] = EMPTY_VALUE;
       inInterval = false;
     } else {
       buffInBoxLow[i] = EMPTY_VALUE;
       buffInBoxHigh[i] = EMPTY_VALUE;
       buffOutBoxLow[i] = EMPTY_VALUE;
       buffOutBoxHigh[i] = EMPTY_VALUE;
     }     
   }
   if (limit > 1) {
     Debug(buffInBoxLow[0], buffOutBoxLow[0], buffInBoxHigh[0], buffOutBoxHigh[0], lastStartIdx, inInterval);
   }
   return(0);
}
//+------------------------------------------------------------------+